/*
 * movie.h -- movie interface header
 * (C)Copyright 2000 by Hiroshi Takekawa
 * This file is part of Enfle.
 *
 * Last Modified: Mon Apr 12 19:39:09 2004.
 * $Id: movie.h,v 1.23 2004/04/18 06:28:13 sian Exp $
 *
 * Enfle is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Enfle is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef _MOVIE_H
#define _MOVIE_H

typedef struct _movie Movie;

#include "utils/libconfig.h"
#include "image.h"
#include "player-extra.h"
#include "demultiplexer.h"
#include "fourcc.h"
#include "video-plugin.h"
#include "videodecoder.h"
#include "audio-plugin.h"
#include "audiodecoder.h"
#include "stream.h"
#include "utils/timer.h"
#include "utils/timer_gettimeofday.h"

typedef enum {
  _STOP,
  _PAUSE,
  _PLAY,
  _RESIZING,
  _UNLOADED
} MovieStatus;

struct _movie {
  void *movie_private;
  Stream *st;
  MovieStatus status;
  ImageType requested_type;
  Timer *timer;
  Demultiplexer *demux;
  int bitrate;

  int has_video;
  int width, height;
  int rendering_width, rendering_height;
  unsigned int current_frame, num_of_frames;
  float framerate;
  int direct_decode;
  unsigned int v_fourcc;
  const char *v_codec_name;
  VideoDecoder *vdec;
  unsigned int out_fourcc;
  int out_bitcount;
  void *video_header;
  void *video_extradata;
  int video_extradata_size;

  int has_audio;
  unsigned int current_sample, num_of_samples;
  AudioFormat sampleformat, sampleformat_actual;
  int channels, channels_actual, samplerate, samplerate_actual;
  char *player_name, *format;
  unsigned int a_fourcc;
  const char *a_codec_name;
  AudioDecoder *adec;
  AudioPlugin *ap;
  int block_align;
  void *audio_extradata;
  int audio_extradata_size;

  /* These are callback functions which may or should be provided by UI. */
  int (*initialize_screen)(VideoWindow *, Movie *, int, int);
  int (*render_frame)(VideoWindow *, Movie *, Image *);
  int (*pause_usec)(unsigned int);

  /* These are methods. */
  void (*unload)(Movie *);
  void (*destroy)(Movie *);

  /* These are implemented by movie plugin. */
  PlayerStatus (*play)(Movie *);
  PlayerStatus (*play_main)(Movie *, VideoWindow *);
  PlayerStatus (*pause_movie)(Movie *);
  PlayerStatus (*stop)(Movie *);
  void (*unload_movie)(Movie *);
};

#define movie_play(m) (m)->play((m))
#define movie_play_main(m, vw) (m)->play_main((m), (vw))
#define movie_pause(m) (m)->pause_movie((m))
#define movie_stop(m) (m)->stop((m))
#define movie_unload(m) (m)->unload((m))
#define movie_resize(m) (m)->status = _RESIZING
#define movie_destroy(m) (m)->destroy((m))

Movie *movie_create(void);

#endif
